/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
/**
 * @author hyassine
 */
dojo.provide("com.ibm.team.scm.web.ui.internal.page.ScmPageController");

dojo.require("dojo.i18n");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("com.ibm.team.process.web.client.projectarea");

dojo.require("com.ibm.team.scm.web.client.util");
dojo.require("com.ibm.team.scm.web.client.internal.ScmClient");

dojo.requireLocalization("com.ibm.team.scm.web.internal", "Messages");

(function() {
	var checkNotNull = net.jazz.ajax.toolbox.lang.checkNotNull;
	var checkNonNegative = net.jazz.ajax.toolbox.lang.checkNonNegative;
	var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var PlatformUI = net.jazz.ajax.ui.PlatformUI;
	
	var ScmUtil = com.ibm.team.scm.web.client.util;
	var ScmClient = com.ibm.team.scm.web.client.internal.ScmClient;
	
	/**
	 * Main application controller method for Jazz source control web UI.  Modules
	 * *outside* of the com.ibm.team.scm.web component should *not* access
	 * methods in this controller directly; rather external modules should update
	 * the browser's URIs using URIs returned from
	 * <code>com.ibm.team.scm.web.client.util</code> helper methods.
	 */
	dojo.declare("com.ibm.team.scm.web.ui.internal.page.ScmPageController", null, {
	
	//These should be set to null when every action is executed except for the viewChangeset action.
	workspaceChangesetPair: {workspaceId: null, changesetId: null}, //Used for navigating to diffs within a change set
	DEFAULT_PAGE_SIZE: 25,
	
	//Internal constructor
	constructor: function(scmView) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.scm.web.internal", "Messages");
		this._currentOperation= null;
		this._view = scmView;
		this._streamList= [];
	},
	
	/*
	 * Creates an operation specified by the given name and id.
	 */
	createOperation: function(/* string */ name, /* string */ id) {
		checkNotNull(name, "name");
		checkNotNull(id, "id");
		return { name: name, id: id };
	},
	
	/*
	 * Marks the start of a long-running operation.
	 * May be called by code within the com.ibm.team.scm.web bundle.
	 */
	startOperation: function(operation) {
		PlatformUI.getWorkbench().showMessage(operation.name);
	},

	/*
	 * Marks the end of a long-running operation.
	 * May be called by code within the com.ibm.team.scm.web bundle.
	 */
	finishOperation: function(operation) {
		PlatformUI.getWorkbench().hideMessageArea();
	},

	//Set the page title with the specified title.
	_setPageTitle: function(title) {
	    PlatformUI.getWorkbench().setTitle(title);
	},
	
	_setHistoryPageSize: function(n) {
		this._view.setHistoryPageSize(n);
	},
	
	_handleError: function(message) {
		this._view.displayError(message);
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// Action Processing Methods
	///////////////////////////////////////////////////////////////////////////////////

	/*
	 * Special method called by the Jazz web ui framework when the URL contains
	 * no 'action' request parameter.
	 * 
	 * Display the main page view for the source control web UI.
	*/
	_showDefaultView: function() {
		this.workspaceChangesetPair.workspaceId = null;
		this.workspaceChangesetPair.changesetId = null;
		this._setPageTitle(this.messages.ScmPageController_0);
		this._view._showMainPage();
	},

	/*
	 * Display active change sets within a contributor's team areas.
	 */
	showTeamChanges: function(kwArgs) {
		this.workspaceChangesetPair.workspaceId = null;
		this.workspaceChangesetPair.changesetId = null;
		
		//Start a new operation
		var op = null;
		op = this.createOperation(this.messages.ScmPageController_27, "view team changes");
		this._currentOperation= op;
		this.startOperation(op);

		var self= this;
		var serviceObject= {
			showTeamChangesHandler: function(changeSets) {
				self._setPageTitle(this.messages.ScmPageController_3);
				if (changeSets) {
					self._displayTeamChanges(changeSets);
				}
				else {
					self._displayTeamChanges("none");
				}
				if (op && op == self._currentOperation) {
					self.finishOperation(op);
					self._currentOperation= null;
				}
			},
			error: function(error) {
				if (error.message != "") {
					self._handleError(dojo.string.substitute(this.messages.ScmPageController_4, [ error.message , error.status ]));
				}
				else {
					self._handleError(this.messages.ScmPageController_5);
				}
				if (op && op == self._currentOperation) {
						self.finishOperation(op);
						self._currentOperation= null;
				}
			}
		};
		
		var currentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
				
		var contrib = com.ibm.team.repository.web.client.session.getAuthenticatedContributor();
		var ownerItemId = contrib.itemId;

		ScmClient.getTeamChanges(new ServiceResponseHandler(serviceObject, "showTeamChangesHandler", "error"), {projectAreaItemId: currentProjectArea.itemId, ownerItemId: ownerItemId});
	},

	/*
	 * Display the selected stream's components for browsing.
	 */
	browseStream: function(kwArgs) {
		checkNotNull(kwArgs, "kwArgs");
		checkNotNull(kwArgs.id, "kwArgs.id");
		
		this.workspaceChangesetPair.workspaceId = null;
		this.workspaceChangesetPair.changesetId = null;
		
		var streamId = kwArgs.id;
		
		//Start a new operation
		var op = null;
		op = this.createOperation(this.messages.ScmPageController_28, "browse stream");
		this._currentOperation= op;
		this.startOperation(op);

		var self= this;
		var serviceObject= {
			browseStreamHandler: function(streamComponents) {
				checkNotNull(streamComponents, "streamComponents");
				if (streamComponents) {
					self._setPageTitle(streamComponents.workspace.name);
					self._displayStream(streamComponents);	//Display the stream components
					if (op && op == self._currentOperation) {
						self.finishOperation(op);
						self._currentOperation= null;
					}
				} else {
					alert(dojo.string.substitute(this.messages.ScmPageController_6, [ streamId ]));
				}
			},
			error: function(error) { 
				if (error.message != "") {
					self._handleError(dojo.string.substitute(this.messages.ScmPageController_7, [ error.message, error.status ]));
				}
				else {
					self._handleError(this.messages.ScmPageController_8);
				}
				if (op && op == self._currentOperation) {
						self.finishOperation(op);
						self._currentOperation= null;
				}
			}
		};
		
		ScmClient.getWorkspace(new ServiceResponseHandler(serviceObject, "browseStreamHandler", "error"), {itemId: streamId});
	},
	
	/*
	 * Display the selected workspace's components for browsing.
	 */
	browseWorkspace: function(kwArgs) {
		checkNotNull(kwArgs, "kwArgs");
		checkNotNull(kwArgs.id, "kwArgs.id");
		
		this.workspaceChangesetPair.workspaceId = null;
		this.workspaceChangesetPair.changesetId = null;

		var workspaceId = kwArgs.id;
		
		var op = null;
		op = this.createOperation(this.messages.ScmPageController_29, "browse workspace");
		this._currentOperation= op;
		this.startOperation(op);

		var self= this;
		var serviceObject= {
			browseWorkspaceHandler: function(workspaceComponents) {
				checkNotNull(workspaceComponents, "workspaceComponents");
				if (workspaceComponents) {
					self._setPageTitle(workspaceComponents.workspace.name);
					self._displayWorkspace(workspaceComponents);	//Display the workspace components
					if (op && op == self._currentOperation) {
						self.finishOperation(op);
						self._currentOperation= null;
					}
				} else {
					alert(dojo.string.substitute(this.messages.ScmPageController_9, [ workspaceId ]));
				}
			},
			error: function(error) { 
				if (error.message != "") {
					self._handleError(dojo.string.substitute(this.messages.ScmPageController_10, [ error.message , error.status ]));
				}
				else {
					self._handleError(this.messages.ScmPageController_11);
				}
				if (op && op == self._currentOperation) {
						self.finishOperation(op);
						self._currentOperation= null;
				}
			}
		};

		ScmClient.getWorkspace(new ServiceResponseHandler(serviceObject, "browseWorkspaceHandler", "error"), {itemId: workspaceId});
	},
	
	
	/*
	 * Display the component/folder for browsing, or the file to be viewed depending on the specified path.
	 */
	browseElement: function(kwArgs) {
		checkNotNull(kwArgs, "kwArgs");
		checkNotNull(kwArgs.workspaceItemId, "kwArgs.workspaceItemId");
		checkNotNull(kwArgs.componentItemId, "kwArgs.componentItemId");
		checkNotNull(kwArgs.path, "kwArgs.path");
		
		this.workspaceChangesetPair.workspaceId = null;
		this.workspaceChangesetPair.changesetId = null;
		
		var workspaceItemId = kwArgs.workspaceItemId;
		var componentItemId = kwArgs.componentItemId;
		var path = kwArgs.path;
				
		var op = null;
		op = this.createOperation(this.messages.ScmPageController_30, "browse item, view content");
		this._currentOperation= op;
		this.startOperation(op);

		var self= this;
		var serviceObject= {
			browseElementHandler: function(subElements) {
				self._setPageTitle(subElements.workspace.name);
				if(path!="/") {
					self._displayElement(subElements, "item"); //File or folder
				}
				else {
					self._displayElement(subElements, "component");	//Component
				}
				if (op && op == self._currentOperation) {
						self.finishOperation(op);
						self._currentOperation= null;
				}
			},
			error: function(error) { 
				if (error.message != "") {
					self._handleError(dojo.string.substitute(this.messages.ScmPageController_12, [ error.message , error.status ]));
				}
				else {
					self._handleError(this.messages.ScmPageController_13);
				}
				if (op && op == self._currentOperation) {
						self.finishOperation(op);
						self._currentOperation= null;
				}
			}
		};
		var WORKSPACE_PREFIX = "itemOid/com.ibm.team.scm.Workspace/";
		var COMPONENT_PREFIX = "itemOid/com.ibm.team.scm.Component/";
		
		ScmClient.getVersionable(new ServiceResponseHandler(serviceObject, "browseElementHandler", "error"), {workspace: WORKSPACE_PREFIX + workspaceItemId, component: COMPONENT_PREFIX + componentItemId}, true, path);
	},
	
	/*
	 * Display the component history (list of change sets).
	 */
	viewComponentHistory: function(kwArgs) {
		checkNotNull(kwArgs, "kwArgs");
		checkNotNull(kwArgs.workspaceItemId, "kwArgs.workspaceItemId");
		checkNotNull(kwArgs.componentItemId, "kwArgs.componentItemId");
		
		this.workspaceChangesetPair.workspaceId = null;
		this.workspaceChangesetPair.changesetId = null;
		
		var workspaceItemId = kwArgs.workspaceItemId;
		var componentItemId = kwArgs.componentItemId;
		
		var op = null;
		op = this.createOperation(this.messages.ScmPageController_31, "view component history");
		this._currentOperation= op;
		this.startOperation(op);

		var self= this;
		var serviceObject= {
			viewComponentHistoryHandler: function(historyDTO) {
				checkNotNull(historyDTO, "historyDTO");
				if (historyDTO) {
					self._setPageTitle(historyDTO.workspace.name);
					self._displayComponentHistory(historyDTO); //Display the component history
					if (op && op == self._currentOperation) {
						self.finishOperation(op);
						self._currentOperation= null;
					}
				}else {
					alert(dojo.string.substitute(this.messages.ScmPageController_14, [ componentItemId ]));
				}
			},
			error: function(error) { 
				if (error.message != "") {
					self._handleError(dojo.string.substitute(this.messages.ScmPageController_15, [ error.message , error.status ]));
				}
				else {
					self._handleError(this.messages.ScmPageController_16);
				}
				if (op && op == self._currentOperation) {
						self.finishOperation(op);
						self._currentOperation= null;
				}
			}
		};
		
		//If a paging size n is specified
		if (kwArgs.n) {
			this._setHistoryPageSize(kwArgs.n);
			//"Older" link in the history view is selected.
			if (kwArgs.first) {
				ScmClient.getHistory(new ServiceResponseHandler(serviceObject, "viewComponentHistoryHandler", "error"), {
					workspaceItemId: workspaceItemId,
					componentItemId: componentItemId,
					first: kwArgs.first,
					n: kwArgs.n
				});
			}
			//"Newer" link in the history view is selected.
			else if (kwArgs.last) {
				ScmClient.getHistory(new ServiceResponseHandler(serviceObject, "viewComponentHistoryHandler", "error"), {
					workspaceItemId: workspaceItemId,
					componentItemId: componentItemId,
					last: kwArgs.last,
					n: kwArgs.n
				});
			}
			//Default history link selection.
			else {
				ScmClient.getHistory(new ServiceResponseHandler(serviceObject, "viewComponentHistoryHandler", "error"), {
					workspaceItemId: workspaceItemId,
					componentItemId: componentItemId,
					n: kwArgs.n
				});
			}
		}
		else {
			//Use the default page size.
			this._setHistoryPageSize(DEFAULT_PAGE_SIZE);
			//"Older" link in the history view is selected.
			if (kwArgs.first) {
				ScmClient.getHistory(new ServiceResponseHandler(serviceObject, "viewComponentHistoryHandler", "error"), {
					workspaceItemId: workspaceItemId,
					componentItemId: componentItemId,
					first: kwArgs.first
				});
			}
			//"Newer" link in the history view is selected.
			else if (kwArgs.last) {
				ScmClient.getHistory(new ServiceResponseHandler(serviceObject, "viewComponentHistoryHandler", "error"), {
					workspaceItemId: workspaceItemId,
					componentItemId: componentItemId,
					last: kwArgs.last
				});
			}
			//Default history link selection.
			else {
				ScmClient.getHistory(new ServiceResponseHandler(serviceObject, "viewComponentHistoryHandler", "error"), {
					workspaceItemId: workspaceItemId,
					componentItemId: componentItemId
				});
			}
		}
		
		/* //Commented out until gateway support for component history paging is established
		   //Note that false refers to "not component history"
		var WORKSPACE_PREFIX = "itemOid/com.ibm.team.scm.Workspace/";
		var COMPONENT_PREFIX = "itemOid/com.ibm.team.scm.Component/";
		
		//If a paging size n is specified
		if (kwArgs.n) {
			this._setHistoryPageSize(kwArgs.n);
			//"Older" link in the history view is selected.
			if (kwArgs.first) {
				ScmClient.getHistory(new ServiceResponseHandler(serviceObject, "viewComponentHistoryHandler", "error"), {
					workspace: WORKSPACE_PREFIX + workspaceItemId,
					component: COMPONENT_PREFIX + componentItemId,
					first: kwArgs.first,
					n: kwArgs.n}, true);
			}
			//"Newer" link in the history view is selected.
			else if (kwArgs.last) {
				ScmClient.getHistory(new ServiceResponseHandler(serviceObject, "viewComponentHistoryHandler", "error"), {
					workspace: WORKSPACE_PREFIX + workspaceItemId,
					component: COMPONENT_PREFIX + componentItemId,
					last: kwArgs.last,
					n: kwArgs.n}, true);
			}
			//Default history link selection.
			else {
				ScmClient.getHistory(new ServiceResponseHandler(serviceObject, "viewComponentHistoryHandler", "error"), {
					workspace: WORKSPACE_PREFIX + workspaceItemId,
					component: COMPONENT_PREFIX + componentItemId,
					n: kwArgs.n}, true);
			}
		}
		else {
			//Use the default page size.
			this._setHistoryPageSize(DEFAULT_PAGE_SIZE);
			//"Older" link in the history view is selected.
			if (kwArgs.first) {
				ScmClient.getHistory(new ServiceResponseHandler(serviceObject, "viewComponentHistoryHandler", "error"), {
					workspace: WORKSPACE_PREFIX + workspaceItemId,
					component: COMPONENT_PREFIX + componentItemId,
					first: kwArgs.first}, true);
			}
			//"Newer" link in the history view is selected.
			else if (kwArgs.last) {
				ScmClient.getHistory(new ServiceResponseHandler(serviceObject, "viewComponentHistoryHandler", "error"), {
					workspace: WORKSPACE_PREFIX + workspaceItemId,
					component: COMPONENT_PREFIX + componentItemId,
					last: kwArgs.last}, true);
			}
			//Default history link selection.
			else {
				ScmClient.getHistory(new ServiceResponseHandler(serviceObject, "viewComponentHistoryHandler", "error"), {
					workspace: WORKSPACE_PREFIX + workspaceItemId,
					component: COMPONENT_PREFIX + componentItemId}, true);
			}
		}*/
	},
	
	/*
	 * Display the history (list of change sets) of an item - file or folder.
	 */
	viewItemHistory: function(kwArgs) {
		checkNotNull(kwArgs, "kwArgs");
		checkNotNull(kwArgs.workspaceItemId, "kwArgs.workspaceItemId");
		checkNotNull(kwArgs.componentItemId, "kwArgs.componentItemId");
		checkNotNull(kwArgs.oid, "kwArgs.oid");
		
		this.workspaceChangesetPair.workspaceId = null;
		this.workspaceChangesetPair.changesetId = null;
		
		var workspaceItemId = kwArgs.workspaceItemId;
		var componentItemId = kwArgs.componentItemId;
		var oid = kwArgs.oid;
		
		var op = null;
		op = this.createOperation(this.messages.ScmPageController_32, "view item history");
		this._currentOperation= op;
		this.startOperation(op);

		var self= this;
		var serviceObject= {
			viewItemHistoryHandler: function(historyDTO) {
				checkNotNull(historyDTO, "historyDTO");
				if (historyDTO) {
					self._setPageTitle(historyDTO.workspace.name);
					self._displayItemHistory(historyDTO); //Display the item history
					if (op && op == self._currentOperation) {
						self.finishOperation(op);
						self._currentOperation= null;
					}
				}else {
					alert(dojo.string.substitute(this.messages.ScmPageController_14, [ oid ]));
				}
			},
			error: function(error) { 
				if (error.message != "") {
					self._handleError(dojo.string.substitute(this.messages.ScmPageController_17, [ error.message , error.status ]));
				}
				else {
					self._handleError(this.messages.ScmPageController_18);
				}
				if (op && op == self._currentOperation) {
						self.finishOperation(op);
						self._currentOperation= null;
				}
			}
		};
		
		ScmClient.getHistory(new ServiceResponseHandler(serviceObject, "viewItemHistoryHandler", "error"), {
			workspaceItemId: workspaceItemId,
			componentItemId: componentItemId,
			oid: oid
		});
		
		/* //Commented out until gateway support for component history paging is established
		   //Note that false refers to "not component history"
		var WORKSPACE_PREFIX = "itemOid/com.ibm.team.scm.Workspace/";
		var COMPONENT_PREFIX = "itemOid/com.ibm.team.scm.Component/";
		ScmClient.getHistory(new ServiceResponseHandler(serviceObject, "viewItemHistoryHandler", "error"), {
			workspace: WORKSPACE_PREFIX + workspaceItemId,
			component: COMPONENT_PREFIX + componentItemId,}, 
			false, oid);
		*/
	},

	/*
	 * Display the selected change set (summary and diffs).
	 */
	viewChangeSet: function(kwArgs) {
		checkNotNull(kwArgs, "kwArgs");
		checkNotNull(kwArgs.workspaceItemId, "kwArgs.workspaceItemId");
		checkNotNull(kwArgs.changeSetItemId, "kwArgs.changeSetItemId");
		
		var workspaceItemId = kwArgs.workspaceItemId;
		var changeSetItemId = kwArgs.changeSetItemId;
		var fromHistory = kwArgs.fromHistory;
		
		//If a diff anchor is specified and the browser is currently displaying the change set, simply scroll the page to the specified diff.
		if (kwArgs.diffId && workspaceItemId==this.workspaceChangesetPair.workspaceId && changeSetItemId==this.workspaceChangesetPair.changesetId) {
			this._scrollToDiff(kwArgs.diffId);
		}
		else {
			this.workspaceChangesetPair.workspaceId = workspaceItemId;
			this.workspaceChangesetPair.changesetId = changeSetItemId;
			
			var op = null;
			op = this.createOperation(this.messages.ScmPageController_33, "view change set");
			this._currentOperation = op;
			this.startOperation(op);
			
			var self = this;
			var serviceObject = {
				viewChangeSetHandler: function(changeSetDTO){
					checkNotNull(changeSetDTO, "changeSetDTO");
					if (changeSetDTO) {
						self._setPageTitle(changeSetDTO.workspace.name);
						self._displayChangeSet(changeSetDTO, fromHistory); //Display the change set
						if(kwArgs.diffId) {
							self._scrollToDiff(kwArgs.diffId);
						}
						if (op && op == self._currentOperation) {
							self.finishOperation(op);
							self._currentOperation = null;
						}
					}
					else {
						alert(dojo.string.substitute(this.messages.ScmPageController_14, [ componentItemId ]));
					}
				},
				error: function(error){
				if (error.message != "") {
					self._handleError(dojo.string.substitute(this.messages.ScmPageController_19, [ error.message , error.status ]));
				}
				else {
					self._handleError(this.messages.ScmPageController_20);
				}
					if (op && op == self._currentOperation) {
						self.finishOperation(op);
						self._currentOperation = null;
					}
				}
			};
				
			ScmClient.getChangeSet(new ServiceResponseHandler(serviceObject, "viewChangeSetHandler", "error"), {
				workspaceItemId: workspaceItemId,
				changeSetItemId: changeSetItemId
			});
			
			/* //Commented out until gateway support for changeset information is established
			var WORKSPACE_PREFIX = "itemOid/com.ibm.team.scm.Workspace/";
			ScmClient.getChangeSet(new ServiceResponseHandler(serviceObject, "viewChangeSetHandler", "error"), {
			workspace: WORKSPACE_PREFIX + workspaceItemId}, 
			changeSetItemId);
			*/
		}
	},
	
	/*
	 * Display the selected change set item (folders, before and after states of files).
	 */
	viewChangeSetItem: function(kwArgs) {
		checkNotNull(kwArgs, "kwArgs");
		checkNotNull(kwArgs.workspaceItemId, "kwArgs.workspaceItemId");
		checkNotNull(kwArgs.componentItemId, "kwArgs.componentItemId");
		checkNotNull(kwArgs.itemType, "kwArgs.itemType");
		checkNotNull(kwArgs.itemId, "kwArgs.itemId");
		
		this.workspaceChangesetPair.workspaceId = null;
		this.workspaceChangesetPair.changesetId = null;
		
		var workspaceItemId = kwArgs.workspaceItemId;
		var componentItemId = kwArgs.componentItemId;
		var itemType = kwArgs.itemType;
		var itemId = kwArgs.itemId;
		var oid = /*itemType + "/" +*/ itemId;	//Construct the oid
		
		if(kwArgs.stateId) {
			var stateId = kwArgs.stateId;
			oid = oid + "/" + stateId;
		}

		var op = null;
		op = this.createOperation(this.messages.ScmPageController_34, "Open file or folder in changeset");
		this._currentOperation= op;
		this.startOperation(op);

		var self= this;
		var serviceObject= {
			viewChangeSetItemHandler: function(contentDTO) {
				checkNotNull(contentDTO, "contentDTO");
				if (contentDTO) {
					self._setPageTitle(contentDTO.workspace.name);
					if(kwArgs.stateId) {
						self._displayStateContent(contentDTO); //Display the file content at the specified state
					}
					else {
						self._displayElement(contentDTO, "item"); //Display the content of the current state of the file
					}
					if (op && op == self._currentOperation) {
						self.finishOperation(op);
						self._currentOperation= null;
					}
				}else {
					if(kwArgs.stateId) {
						alert(dojo.string.substitute(this.messages.ScmPageController_21, [ stateId , itemId ]));
					}
					else {
						alert(dojo.string.substitute(this.messages.ScmPageController_22, [ itemId ]));
					}
				}
			},
			error: function(error) { 
				if (error.message != "") {
					self._handleError(dojo.string.substitute(this.messages.ScmPageController_23, [ error.message ]));
				}
				else {
					self._handleError(this.messages.ScmPageController_24);
				}
				if (op && op == self._currentOperation) {
						self.finishOperation(op);
						self._currentOperation= null;
				}
			}
		};
		
		var WORKSPACE_PREFIX = "itemOid/com.ibm.team.scm.Workspace/";
		var COMPONENT_PREFIX = "itemOid/com.ibm.team.scm.Component/";
		ScmClient.getVersionable(new ServiceResponseHandler(serviceObject, "viewChangeSetItemHandler", "error"), {workspace: WORKSPACE_PREFIX + workspaceItemId, component: COMPONENT_PREFIX + componentItemId}, false, oid);
	},
	
	/*
	 * Display the snapshots of a workspace or stream.
	 */
	viewSnapshots: function(kwArgs) {
		checkNotNull(kwArgs, "kwArgs");
		checkNotNull(kwArgs.workspaceItemId, "kwArgs.workspaceItemId");
		
		this.workspaceChangesetPair.workspaceId = null;
		this.workspaceChangesetPair.changesetId = null;
		
		var workspaceItemId = kwArgs.workspaceItemId;
		
		var op = null;
		op = this.createOperation(this.messages.ScmPageController_35, "view snapshots");
		this._currentOperation= op;
		this.startOperation(op);

		var self= this;
		var serviceObject= {
			viewSnapshotsHandler: function(baselineSets) {
				if (baselineSets) {
					self._displaySnapshots(baselineSets); //Display the baseline sets
					if (op && op == self._currentOperation) {
						self.finishOperation(op);
						self._currentOperation= null;
					}
				}else {
					self._displaySnapshots(null); //Display an empty list of baseline sets
					if (op && op == self._currentOperation) {
						self.finishOperation(op);
						self._currentOperation= null;
					}
				}
			},
			error: function(error) { 
				if (error.message != "") {
					self._handleError(dojo.string.substitute(this.messages.ScmPageController_25, [ error.message , error.status ]));
				}
				else {
					self._handleError(this.messages.ScmPageController_26);
				}
				if (op && op == self._currentOperation) {
						self.finishOperation(op);
						self._currentOperation= null;
				}
			}
		};
		
		ScmClient.getBaselineSets(new ServiceResponseHandler(serviceObject, "viewSnapshotsHandler", "error"), {workspaceItemId: workspaceItemId});
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// View Update Methods
	///////////////////////////////////////////////////////////////////////////////////
	
	//Scroll to the specified diff within a change set.
	_scrollToDiff: function(diffId) {
		this._view.scrollToDiffInChangeset(diffId);
	},
	
	//Update the view to display the change set.
	_displayChangeSet: function(changeSetDTO, fromHistory) {
		this._view.showChangeSetView(changeSetDTO, fromHistory);
	},
	
	//Update the view to display the stream components.
	_displayStream: function(streamComponents) {
		this._view.showStreamView(streamComponents);
	},

	//Update the view to display the workspace components.
	_displayWorkspace: function(workspaceComponents) {
		this._view.showWorkspaceView(workspaceComponents);
	},
	
	//Update the view to display the item (folder or file view).
	_displayElement: function(subElements, elementType) {
		this._view.showElementView(subElements, elementType);
	},
	
	//Update the view to display the component history.
	_displayComponentHistory: function(historyDTO) {
		this._view.showComponentHistory(historyDTO);
	},
	
	//Update the view to display the item history.	
	_displayItemHistory: function(historyDTO) {
		this._view.showItemHistory(historyDTO);
	},
	
	//Update the view to display the active team change sets.
	_displayTeamChanges: function(changeSets) {
		this._view.showTeamChanges(changeSets);
	},

	//Update the view to display the stream/workspace snapshots.
	_displaySnapshots: function(baselineSets) {
		this._view.showSnapshots(baselineSets);
	},
	
	//Update the view to display the file/folder at the specified state.
	_displayStateContent: function(contentDTO) {
		this._view.showStateContent(contentDTO);
	}
	
});
})();
